#ifndef JWT_H
#define JWT_H
#include <string>
#include "resources.h"

namespace http::auth::jwt {
    extern std::string secretKey;

    constexpr const char* EMPTY_AUTH_COOKIE = "auth=;Path=/; Max-Age=86400; HttpOnly; SameSite=Lax";;

    void generateSecretKey();

    /**
     * Упрощенная реализация JWT (Json Web Token). Токен имеет вид: `{ username | base64 }.{ signature | base64 }`.
     * Сигнатура вычисляется следующим образом: `SHA256(username + secret)`.
     * Имя cookie: `auth`.
     */
    class Jwt {
        std::string payload;
        std::string signature;
    public:
        static Jwt fromCookies(const std::string& cookie);
        static Jwt fromUser(const std::string& User);

        bool isValid();

        std::string getUsername();

        std::string asCookie();

        ~Jwt();
    };
}

#endif //JWT_H
